import java.rmi.*;
import net.jini.lease.*;
import net.jini.core.lookup.*;
import net.jini.discovery.*;
import net.jini.core.discovery.*;
import net.jini.core.event.*;
import net.jini.core.lease.*;
import java.util.*;
import java.io.Serializable;
import media.network.*;
import media.common.*;

public class ClientPart implements Serializable, DiscoveryListener{

    //Private Variable Declaration
    private LookupDiscovery lookup              = null; //looks for all available lookup servers
    private ServiceTemplate template            = null; //the template that is searched for
    private VideoConferenceListenerInterface find  = null; //the interface that informs client of new services
    private Network net                      = null; //the network that manages the tcp and rtp connections
    private LeaseRenewalManager leaser          = null; //grants the lease

    /**
      *  A simple Jini client that searches for the
      *  different lookup servers and downloads the
      *  template representing the service.
      */
 
    public ClientPart(Network net){
	this.net = net;
	this.leaser = new LeaseRenewalManager( );
	System.setSecurityManager(new RMISecurityManager( ));
	Class[ ] classes = new Class[ ]{VideoConference.class};
	template = new ServiceTemplate(null, classes, null);
	try{
	    find = new VideoConferenceListener(template,net);       
	    lookup = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);	    
	}catch(java.io.IOException e){
	    System.err.println(e.toString( ));
	    System.out.println("oops! couldn't create lookup");
	}
	
	lookup.addDiscoveryListener(this);
    }

    public synchronized void discovered(DiscoveryEvent e){

	ServiceRegistrar[ ] registrars = e.getRegistrars( );
	for(int i =0; i< registrars.length; i++){
	    //get the first lookup service
	    try{
	    ServiceRegistrar registrar = registrars[i];
	    VideoConference video = null;	  
	    String hostInfo =null;

	    /**
	      * get all matching services from this server
	      */
	    ServiceMatches services = registrar.lookup(template,Integer.MAX_VALUE);
	    /**
	      * get the locator associated with this service
	      */
	    LookupLocator locator = registrar.getLocator( );
	    /**
	      *  get the information from each one of the services
	      */
	    for(int j =0; j< services.items.length; j++){
		if(services.items[j].service != null){
		    video = (VideoConference) services.items[j].service;
		    hostInfo = video.revealHost( );
		    User newUser = new User(hostInfo);		  
		    net.addUser(newUser);		 
		}//end if
	    }//end for	   
	    /**
	      * notify me of any other services that come in afterwards	   
	      */
	    EventRegistration ereg = registrar.notify(template,
						      ServiceRegistrar.TRANSITION_NOMATCH_MATCH,
						      find,null,Lease.ANY);	   
	    }catch(Exception ex){
		System.err.println(ex.toString( ));
		System.out.println("OOPS");
	    }//end catch
	}//end outer for
    }//end method discovered

    public synchronized void discarded(DiscoveryEvent e){
	ServiceRegistrar[] registrars = e.getRegistrars( );
	for(int i=0; i<registrars.length;i++){
	/**
	 * a condition where a Service Registrar 
	 * leaves the vicinity.We cannot move the 
	 * users from the lookup server becuase they
	 * may actually be present on the other 
	 * lookup servers.
	 */
 	    try{
 		ServiceRegistrar registrar = registrars[i];
		LookupLocator locator = registrar.getLocator( );
		System.out.println("The Lookup Service "+locator.getHost( )+" has left.");
	    }catch(Exception ex){
		System.out.println("Registrar could not leave");
	    }
	}//end for
    }//end method
}//end of class ClientPart
