
package media.network;

import javax.swing.*;
import javax.media.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.media.format.*;
import javax.media.control.*;
import javax.media.protocol.*;
import javax.media.rtp.rtcp.*;
import javax.media.rtp.event.*;
import javax.media.datasink.*;
import javax.media.rtp.*;
import javax.media.rtp.SessionManager;
import media.gui.*;
import media.config.*;

/**
 *
 * Title:        AudioReceiver.java<br>
 * Description:  Awaits for connection from some host and creates a player<br>
 * Copyright:    Konrad Rzeszutek<br>
 * Company:      darnok.dhs.org<p>
 * Version:      $Id: AudioReceiver.java,v 1.3 2000/11/28 03:41:28 konrad Exp $ <br>
 *
 * Initalizes the RTP seesion manager and awaits for connection. Listens
 * only on the Audio Port and formats the input stream to GSM (Audio).
 * This class gets its date (who to speak with, etc) from the
 * class config.Config
 */
public class AudioReceiver implements ControllerListener,
                                      ReceiveStreamListener {

  public Player audio_player   = null;
  private Component controlComponent    = null;

  private OcularBase gui        = null;
  private Config config         = null;
  private SessionManager m      = null;

  public final static int OK    = 94321;
  public final static int ERROR = 128321;
  private boolean audioPlayerRealized = false; //boolean if audio player realized

  /**
   * Initializes the Audio receiver (which awaits for connections from
   * the host specified in config).<br><b>Parameters</b>:<br>
   *   gui != null<br>
   *   config != null
   */
  public AudioReceiver (OcularBase gui, Config config) {

    this.gui = gui;
    this.config = config;
    config.write("Audio receiver started");
  }

  /**
   * Starts the receiver. Without starting this method, no
   * data will be received at all.
   */
  public synchronized int  startAudioReceiver() {

    if (resetAudio() == OK)
        m = createAudioManager();
      else
        m = null;

      if (m == null)
        return ERROR;

      return OK;
  }
  /**
   * Resets RTP incoming video receiver to its initial state.
   */
  public synchronized int  resetAudio() {

      int return_state = OK;

      if (audio_player == null) {
	  if (m != null) {
	      m.removeReceiveStreamListener(this);
	      m.removeAllPeers();
	      m.closeSession("bye!");
	  }	  
	  return return_state;
      }
      try {
        audio_player.stop();
        audio_player.deallocate();
        audioPlayerRealized = false;
      } catch (Exception e) {
        config.write(e.getMessage());
        return_state = ERROR;
      }
      audio_player = null;
      if (m != null) {
        m.removeAllPeers();
        m.closeSession("I want to close");
      }
      
      return return_state;
  }

  /**
   * Initalizes the RTPSessionManager and setups for any connection.
   */
  private SessionManager createAudioManager() {


    SessionManager mgr = (SessionManager)new com.sun.media.rtp.RTPSessionMgr();

    /* That means something really bad happend */

    if (mgr == null) return null;

    /* Go thru the format and change it */

    mgr.addFormat(new AudioFormat(config.DEFAULT_AUDIO_ENC),1);
    mgr.addReceiveStreamListener(this);

    SessionAddress localaddr = new SessionAddress();

    try{
       InetAddress destaddr = InetAddress.getByName(config.RECV_HOSTNAME);
       gui.write("Audio receiving data from : "+destaddr);
       mgr.initSession(localaddr,
                         null,
                         0.05,
                         0.25);

       SessionAddress sessaddr = new SessionAddress(destaddr,
                                                    config.RECV_AUD_PORT,
                                                    destaddr,
                                                    config.RECV_AUD_PORT + 1);
       mgr.startSession(sessaddr,config.TTL,null);

   } catch (Exception e) {
       config.write("Couldn't initialize Audio RTP");
       return null;
   }
   return mgr;
  }

  /*
   * This dude is awaken whenever a incoming stream is comming.
   *
   * Perfect place to active the player.
   */

 public void update( ReceiveStreamEvent event) {



   // find the sourceRTPSM for this event
   SessionManager source = (SessionManager)event.getSource();
   //gui.write("A_RTP:"+source);
   // create a new player if a new recvstream is detected
   if (event instanceof NewReceiveStreamEvent) {

       String cname = "";
       ReceiveStream stream = null;

       try {
            // get a handle over the ReceiveStream
            stream =((NewReceiveStreamEvent)event)
                               .getReceiveStream();

            Participant part = stream.getParticipant();
            if (part != null) cname = part.getCNAME();
            // get a handle over the ReceiveStream datasource

            DataSource dsource = stream.getDataSource();
            gui.write("Incoming AudioRTP session from: "+cname);

            // create a player by passing datasource to the
            // Media Manager
            audio_player = javax.media.Manager.createPlayer(dsource);

       } catch (Exception e) {
            System.err.println("NewReceiveStreamEvent exception "
                                          + e.getMessage());
            return;
       }

       if (audio_player == null) return;

       audio_player.addControllerListener(this);
       audio_player.realize();
   }
  }

   public synchronized void controllerUpdate(ControllerEvent event)
   {
             // If we're getting messages from a dead player,
             // just leave
       //gui.write("A_RTP:"+event+"");

      if (audio_player == null) return;

      // When the player is Realized, get the
      // control components and add them to the OcularBase (or not)?

      if (event instanceof RealizeCompleteEvent)
      {
          if (audio_player.getState() == Controller.Realized)
            if ((controlComponent = audio_player.getControlPanelComponent()) != null) {
		//gui.setupConnectedAudio(controlComponent);
		audio_player.start();
		audioPlayerRealized = true;

          }
        }
  }
    /**
     * Return if audio player is realized
     */
    public boolean isRealized(){
	return audioPlayerRealized; 
    }

}






