package media.network;

import java.io.*;
import java.util.*;
import java.net.*;


public class PongThread extends Thread {

    private TCPClient tcp = null; 
    private boolean isPonging = false;
    private ObjectInputStream input =  null;
    private ObjectOutputStream output = null;
    private Socket socket = null;

    /**
     * Just a simple Threaded class which sleeps every 60 seconds,
     * calls TCPClient's sendPing() method, and then sleeps again. It
     * does that until this class method <i>stop</i> is called.
     */
    public PongThread(TCPClient tcp,ObjectInputStream in, ObjectOutputStream out, Socket s) {
	this.tcp = tcp;
	this.input = in;
	this.output = out;
	this.socket = socket;
	
	isPonging = true;
    }//end constructor

    public void changePonging( ){
	isPonging = false;
    }

    public void run () {

	while (isPonging) {
	    try {
		if(isPonging){
		    sendPong( );
		    tcp.net.config.error("PONG!");
		}
		this.sleep(tcp.net.config.INTERVAL);
		receivePing();	
	    }catch(IOException e){
		/**
		 *  Tell that the ping was not
		 *  received
		 */

		tcp.net.config.error("PONG died b/c of "+e.toString ());
		
		/**
		 *  Change present client's status to
		 *  dead and my status to available.
		 */
		tcp.declareDeath( );
		tcp.closeStreams( );

	    } //edn catch IOException
	    catch (Exception e) { //tcp connection breaks
		tcp.net.config.error("Client probably walked away from the connection");
		/**
		 *  Change present client's status to
		 *  dead and my status to available
		 */
		isPonging = false;
		tcp.declareDeath( );		
		tcp.closeStreams( );
	    }//end catch Exception
	}//end while
	tcp.net.config.error("PING PONG finished");
       
    }//end run

    /**
     *  Try to recieve the corresponding ping
     *  to the sent pong.
     */

    private void receivePing( ) throws IOException, ClassNotFoundException, SocketException{
	String in = "";

	in = (String) input.readObject( );
	if(in.equalsIgnoreCase("PING")){
	    isPonging = true;	 
	tcp.net.config.error("PING");
	}
	else if(in.equalsIgnoreCase("BYE")){
	    tcp.keepAlive( );
	    isPonging = false;
	    tcp.net.config.error("BYE recieved");	   
	    tcp.closeStreams( );	  
	}
	else{ //the string is not ping
	    isPonging = false;
 	    throw new IOException("Ping not received");
	}
    }//end recievePing( )

    /**
     *  Send the pong message to the client
     *  on the other side.
     */

    private void sendPong( )throws IOException, SocketException{
	output.writeObject("PONG");
	output.flush( );
    }

}//end PongThread class


















