package media.network;

import java.io.*;
import java.util.*;
import java.net.*;
import media.common.*;
import media.config.*;
public class TCPClient extends Thread{

    //Private Variable Declaration
    public  Network net;
    private Socket socket;
    private boolean connected;
    private PongThread pong = null;    
    private User me = null;
    private User you = null;
    private ObjectOutputStream output = null;
    private boolean done = false;
    private ObjectInputStream input = null;
    private Config config = null;

    public TCPClient(Network net, Socket socket){

	this.net = net;
	this.config = net.giveConfig( );
	this.socket = socket;
	config.error("Socket changed owndership");
    }

    /**
     *  Change the status of myself to <i>available(\i> and
     *  change the status of the client I was connected 
     *  to to <i>dead<\i>
     */

    public void declareDeath(){
	
	me.updateStatus("WAITING");
	you.updateStatus("DEAD");
	config.oldyou = you;
	you = null;
	config.error("I am  "+me.toString( ));
	config.error("You are :"+ config.oldyou.toString());
        //net.gui.resetConnected();
	net.config.STATE = net.config.I_AM_BUSY;
    }

    public void keepAlive( ){
	me.updateStatus("AVAILABLE");
	you.updateStatus("AVAILABLE");
	config.error("I am "+me.toString( ));
	config.error("You are "+you.toString( ));
	net.gui.resetConnected();
	net.disconnect();
	net.config.STATE = net.config.I_AM_AVAILABLE;
    }
    public void closeStreams( ){
	try{
	    // done = true;	    
	    if (pong != null) {
		pong.join();
		// Active session
	    }
	    input.close( );
	    output.close( );
	    socket.close( );
	    input = null;
	    output = null;
	    socket = null;

	}catch(Exception e){
	    config.error("Couldn't close in TCP thread "+e.toString( ));
	}
    }//end method

    public void run( ){	
	boolean goOn = true;
	try{
	    output = new ObjectOutputStream(socket.getOutputStream( ));
	    output.flush( );
	    input = new ObjectInputStream(socket.getInputStream( ));	   
	    String message = (String) input.readObject( );	
	    Users users = net.getHostList( );	
	    config.error("Message is "+message);

	    if(message.equalsIgnoreCase("STATUS")){
		String name = (String) input.readObject( );	
		User userFound = searchFor(name,users);		  
		if(userFound ==null){
		    config.error("Couldnt' find myself in user in database");
		}
		config.error(userFound.getStatus());
		output.writeObject(userFound.getStatus( ));
		output.flush( );	
		closeStreams( );
	    }//end if

	    else if(message.equalsIgnoreCase("CONNECT")){
		String clienthost = (String) input.readObject( );
		String localhost = InetAddress.getLocalHost( ).getHostName( );

		for(int i=0; i<users.size( );i++){
		    User user = (User) users.get(i);
		    if(user.getHostName( ).equalsIgnoreCase(localhost))
			me = user;
		    else if(user.getHostName( ).equalsIgnoreCase(clienthost))
			you = user;
		}//end for
		if (you == null) { you = me; }

		if (net.config.STATE == net.config.I_AM_AVAILABLE){
		    config.error("changing status to available!");
		    me.updateStatus("AVAILABLE");
		    you.updateStatus("AVAILABLE");
		}
		
		String status = me.getStatus( );		
		if (me == you) {
		    output.writeObject("BUZZ OFF");
		    you = null;
		    output.flush( );
		    closeStreams( );
		}
		else if
		(status.equalsIgnoreCase("AVAILABLE")){
		    output.writeObject("WELCOME");
		    output.flush( );
		    
		    /** 
		     *  once user send the ok message 
		     *  we must change his status too
		     */
		    
		    me.updateStatus("BUSY");
		    you.updateStatus("BUSY");
		    config.oldyou = null;
		    String host = (String) input.readObject( );
		    InetAddress ip_addr = InetAddress.getByName(host);
		    String ip_host = ip_addr.getHostAddress();

		    config.RECV_HOSTNAME = config.SEND_HOSTNAME = ip_host;

		    boolean go = true;
		    go = !net.gui.infoDecision("Want to speak with "+you.getName()+"?");
		    if (!go) {
			me.updateStatus("AVAILABLE");
			you.updateStatus("AVAILABLE");
			closeStreams();
		    } else {

		    config.SEND_AUD_PORT = ((Integer) input.readObject( )).intValue( );
		    config.SEND_VID_PORT = ((Integer) input.readObject( )).intValue( );
		    config.error("SendAudPort : "+config.SEND_AUD_PORT);
		    config.error("SendVidPort : "+config.SEND_VID_PORT);
		    config.error("SendHost    : "+config.SEND_HOSTNAME);

		    config.RECV_AUD_PORT = config.RECV_AUD_PORT + 10;
		    config.RECV_VID_PORT = config.RECV_VID_PORT + 10;
		    config.error("RecvAudPort : "+config.RECV_AUD_PORT);
		    config.error("RecvVidPort : "+config.RECV_VID_PORT);
		    System.out.println("RecvHost    : "+config.RECV_HOSTNAME);

		    output.writeObject(me.getHostName( ));
		    output.flush( );
		    output.writeObject(new Integer(config.RECV_AUD_PORT));
		    output.flush( );
		    output.writeObject(new Integer(config.RECV_VID_PORT));
		    output.flush( );
		    config.write("Starting AV listeners");

		    if (goOn && net.audio_receiver.startAudioReceiver() != AudioReceiver.OK)
			if (net.gui.error("Couldn't start Audio RTP listener!")) {
			    goOn = false; }
		    
		    if (goOn && net.video_receiver.startVideoReceiver() != VideoReceiver.OK)
			if (net.gui.error("Couldn't start Video RTP listener!")) {
			    goOn = false; }

		    config.write("Starting AV sender");
		    if (goOn && net.video_sender.startVideoSender() != VideoSender.OK)
		    	if (net.gui.error("Couldn't start Video transmitter!")) {
		    	    goOn = false; }
	    
		    if (goOn && net.audio_sender.startAudioSender() != AudioSender.OK)
			if (net.gui.error("Couldn't start Audio transmitter!")) {
			    goOn =false;; }
		    
		    // Need also the ports (RTP)
		    // "I_AM_AVAILABLE 40912 40922"
		    // "I_AM_AVAILABLE PING 40912 409222"
		    
		    /**
		     *  we want to spawn off a new thread 
		     *  because we want this thread to still
		     *  receive requests from other clients
		     */
		    net.config.STATE = net.config.I_AM_AVAILABLE;
		    pong = new PongThread(this,input,output,socket);
		    pong.start( );	
		    // Change gui status
		    net.gui.state = net.gui.CONNECTED;
		    net.gui.setupPreConnect();
		    }

		}//end if
		
		else if(status.equalsIgnoreCase("BUSY")){ //i.e. this guy is already busy
		    output.writeObject("BUZZ OFF");
		    you = null;
		    output.flush( );
		    closeStreams( );		
		    net.config.STATE = net.config.I_AM_BUSY;
		}//end else if
		else if(status.equalsIgnoreCase("WAITING")){ //i.e.the guy is waiting to reconnect
		    if (config.oldyou == null)
			config.error("old is dead?");
		    if (config.oldyou.getName().equalsIgnoreCase(you.getName())) {
			config.write("Welcome back!");
			output.writeObject("WELCOME BACK");
			output.flush( );
			me.updateStatus("BUSY");
			you.updateStatus("BUSY");
			output.writeObject(me.getHostName( ));
			output.flush( );
			output.writeObject(new Integer(config.RECV_AUD_PORT));
			output.flush( );
			output.writeObject(new Integer(config.RECV_VID_PORT));
			output.flush( );
			pong = new PongThread(this, input, output, socket);
			pong.start();
			net.gui.state = net.gui.CONNECTED;
			/**
			 * The RTP is slow. lets kill it and restart i
			 **/
			net.audio_receiver.resetAudio();
			net.video_receiver.resetVideo();
			net.video_receiver.startVideoReceiver();
			net.audio_receiver.startAudioReceiver();
		    } else
			{
			    output.writeObject("BUZZ OFF");
			    you = null;
			    output.flush();
			    closeStreams();
			    net.config.STATE = net.config.I_AM_BUSY;
			}
		}
		
	    }//end outer else if	   

	    if(pong != null)
		pong.join( );	

	}catch(Exception e){	
	    config.error("Error somewhere in the run method"+e.toString( ));
	}//end catch	

    }//end of run
    
    private User searchFor(String name,Users users){
	config.error("Searching for the user...");
	User found=null;
	for (int i=0; i<users.size( ); i++){
	    User user = (User)users.get(i);
	    if(user.getName( ).equalsIgnoreCase(name)){
		found = user;
		break;
	    }
	}
	return found;
    }//end method searchFor
}//end of TCPClient class











